#ifndef PARACOOBA_WEBSERVER_WEBSERVER_HPP
#define PARACOOBA_WEBSERVER_WEBSERVER_HPP

#ifndef ENABLE_INTERNAL_WEBSERVER
#error "Internal Webserver must be enabled if this header is included!"
#endif

#include "../log.hpp"
#include <boost/asio/io_service.hpp>
#include <boost/version.hpp>
#include <memory>

namespace boost {
namespace system {
class error_code;
}
}

namespace paracooba {
namespace webserver {
class API;
class Webserver
{
  public:
  class HTTPListener;
  class HTTPSession;

  Webserver(ConfigPtr config, LogPtr log, boost::asio::io_service& ioService);
  ~Webserver();

  void run();
  void stop();

  std::string buildLink();

  API* getAPI() { return m_api.get(); }
  boost::asio::io_service& getIOService() { return m_ioService; }
  HTTPListener* getHTTPListener() { return m_httpListener.get(); }

  private:
  void httpSessionClosed(HTTPSession* session);

  ConfigPtr m_config;
  LogPtr m_log;
  Logger m_logger;
  boost::asio::io_service& m_ioService;
  std::unique_ptr<API> m_api;

  std::shared_ptr<HTTPListener> m_httpListener;
};
}
}

#endif
